/* $Id: settings.cpp 29 2009-01-02 13:36:32Z mawerick $ */
/* AOCrayon - Chat color management tool for Anarchy Online
    Copyright (C) 2005 - 2009  Mawerick

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "aocrayon.h"
#include "settings.h"
#include <QtCore>

Settings Prefs;

Settings::Settings()
{
	SettingsMap.clear();
}

void Settings::load()
{
#ifdef ALLOW_LEGACY_PREFS
	if( loadLegacy() )
		return;
#endif

	QSettings settings( PROGRAM ".ini", QSettings::IniFormat );

	QString path = settings.value( "AOGamePath" ).toString();
	qDebug( "Settings::load(): AOPath = %s", qPrintable( path ) );
	set( "AOGamePath", path );

	bool mode = settings.value( "AdvancedMode", false ).toBool();
	qDebug( "Settings::load(): AdvancedMode = %s", mode ? "true" : "false" );
	set( "AdvancedMode", mode );

	QString style = settings.value( "GUIStyle", "plastique" ).toString();
	qDebug( "Settings::load(): GUIStyle = %s", qPrintable( style ) );
	set( "GUIStyle", style );

	QString color = settings.value( "BackgroundColor", "black" ).toString();
	qDebug( "Settings::load(): BGColor = %s", qPrintable( color ) );
	set( "BackgroundColor", color );

	bool firstRun = settings.value( "FirstRun", true ).toBool();
	qDebug( "Settings::load(): FirstRun = %s", firstRun ? "true" : "false" );
	set( "FirstRun", firstRun );
}

#ifdef ALLOW_LEGACY_PREFS
#include "aocolor.h"
bool Settings::loadLegacy()
{
	if( !QFile::exists( PROGRAM ".cfg" ) )
		return false;

	/* set defaults */
	set( "FirstRun", false ); // the file exists, so it can't be first run...
	set( "AOGamePath", "C:\\Program Files\\Funcom\\Anarchy Online" );
	set( "AdvancedMode", false );
	set( "BackgroundColor", "black" );
	set( "GUIStyle", "plastique" );

	/* load the config file */
	QFile file( PROGRAM ".cfg" );
	qDebug( "Settings::loadLegacy(): file: %s", qPrintable( file.fileName() ) );
	if( !file.open( QIODevice::ReadOnly | QIODevice::Text ) )
		return false;

	QTextStream config( &file );
	QString line;
	while( !config.atEnd() )
	{
		line = config.readLine();
		qDebug( "Settings::loadLegacy(): line: %s", qPrintable( line ) );
		QStringList list( line.split( " = " ) );
		if( !list[0].compare( "AODir" ) )
		{
			qDebug( "Settings::loadLegacy(): setting AOGamePath to %s", qPrintable( list[1] ) );
			set( "AOGamePath", list[1] );
		}
		if( !list[0].compare( "AdvancedMode" ) )
		{
			qDebug( "loadSettings::loadLegacy(): setting Advanced mode to %d", list[1].toInt() );
			set( "AdvancedMode", (bool) list[1].toInt() );
		}
		if( !list[0].compare( "BGColor" ) )
		{
			AOColor bgcol;
			bgcol.setColor( list[1] );
			qDebug( "Settings::loadLegacy(): setting BackgroundColor to %s", qPrintable( bgcol.color().name() ) );
			set( "BackgroundColor", bgcol.color().name() );
		}
	}
	file.close();
	file.remove();
	return true;
}
#endif // ALLOW_LEGACY_PREFS

void Settings::save()
{
	QSettings settings( QString( PROGRAM ".ini" ), QSettings::IniFormat );
	QMap<QString,QVariant>::iterator it;
	for( it = SettingsMap.begin(); it != SettingsMap.end(); it++ )
	{
		settings.setValue( it.key(), it.value() );
	}
}

QVariant Settings::get( const QString &name, QVariant defvalue )
{
	return SettingsMap.value( name, defvalue );
}

void Settings::set( const QString &name, QVariant value )
{
	SettingsMap.insert( name, value );
}

QString Settings::cleanPath( const QString &path )
{
	QString clean = QDir::cleanPath( path );
	clean = QDir::convertSeparators( clean );
	return clean;
}
